Attribute VB_Name = "modSystemTree"
Option Explicit

Private mFileSystem As Object

Public Sub handleBuildSystemTree(ByRef tree As TreeView, ByVal topPath As String)
    Static filesBuild As Boolean
    Dim enabledBefore As Boolean
    
    If Not filesBuild Then
        enabledBefore = frmMain.cmdStart.Enabled
        frmMain.cmdStart.Enabled = False
        frmMain.MousePointer = vbHourglass
        frmMain.lblStatus.Caption = "Dateistruktur wird aufgebaut..."
        
        doBuildSystemTree tree, topPath
        filesBuild = True
        calculateSize
        
        frmMain.lblStatus.Caption = "Auswahl, was kopiert werden soll."
        frmMain.MousePointer = vbDefault
        frmMain.cmdStart.Enabled = enabledBefore
    End If
End Sub

Public Sub checkFilesOfChangedDate()
    Dim subNode As node
    Dim repositoryRoot As String
    Dim projectRoot As String
    Dim repositoryPath As String
    Dim projectPath As String
    Dim relativePath As String
    Dim fileSystem As Object
    Dim didChange As Boolean
    Dim changedFiles As Long
    Dim verbose As String
    
    Set fileSystem = CreateObject("Scripting.FileSystemObject")
    
    frmMain.MousePointer = vbHourglass
    frmMain.lblStatus.Caption = "Prfe Dateien mit genderten Datum..."
    
    projectRoot = getPathFrom
    repositoryRoot = getLatestBackupFolder

    changedFiles = 0

    For Each subNode In frmMain.trvFiles.Nodes
        subNode.Checked = False
    Next
    
    For Each subNode In frmMain.trvFiles.Nodes
        If subNode.Tag <> "folder" Then
            relativePath = subNode.FullPath
            projectPath = projectRoot & "\" & relativePath
            repositoryPath = repositoryRoot & "\" & relativePath
            
            If dateChanged(fileSystem, projectPath, repositoryPath) Then
                subNode.Checked = True
                        
                checkAllParents subNode
                changedFiles = changedFiles + 1
            End If
            
        End If
    Next
    
    If changedFiles = 0 Then
        verbose = "Keine Dateien"
    ElseIf changedFiles = 1 Then
        verbose = CStr(changedFiles) & " Datei"
    Else
        verbose = CStr(changedFiles) & " Dateien"
    End If
    frmMain.lblStatus.Caption = verbose & " mit genderten Datum."
    frmMain.MousePointer = vbDefault
End Sub

Public Sub setAllChildren(ByVal newValue As Boolean)
    Dim subNode As node
    
    frmMain.MousePointer = vbHourglass
    For Each subNode In frmMain.trvFiles.Nodes
        If oneParentSelected(subNode) Then
            subNode.Checked = newValue
        End If
    Next
    frmMain.MousePointer = vbDefault
End Sub

Public Sub setAllFileChecksTo(ByVal newValue As Boolean)
    Dim subNode As node
    
    For Each subNode In frmMain.trvFiles.Nodes
        subNode.Checked = newValue
    Next
End Sub

Public Sub filesExpandCollapse(ByVal doExpand As Boolean)
    Dim subNode As node
    
    For Each subNode In frmMain.trvFiles.Nodes
        If Not (subNode.Expanded = doExpand) Then
            subNode.Expanded = doExpand
        End If
    Next
End Sub

Public Sub handleCheckAllParents()
    Dim subNode As node
    
    For Each subNode In frmMain.trvFiles.Nodes
        If subNode.Selected Then
            checkAllParents subNode
        End If
    Next
End Sub

Public Sub selectFileUnique()
    Dim subNode As node
    
    For Each subNode In frmMain.trvFiles.Nodes
        If subNode.Selected Then
            subNode.Checked = True
        Else
            subNode.Checked = False
        End If
    Next
    
    For Each subNode In frmMain.trvFiles.Nodes
        If subNode.Checked Then
            checkAllParents subNode
        End If
    Next
End Sub

Public Sub copyCheckedFiles(ByVal pathFromRelative As String, ByVal pathToRelative As String)
    Dim subNode As node
    Dim fileName As String
    Dim pathFrom As String
    Dim pathTo As String
    Dim pathFolder As String
    Dim success As Boolean
        
    For Each subNode In frmMain.trvFiles.Nodes
        
        If selfAndParentsChecked(subNode) Then
            If subNode.Tag = "folder" Then
                hiliteNode subNode
                subNode.Expanded = True
                success = doCreateFolder(pathToRelative & "\" & subNode.FullPath)
                If success Then
                    loliteNode subNode
                End If
            Else
                hiliteNode subNode
                fileName = subNode.text
                pathFrom = pathFromRelative & "\" & subNode.FullPath
                pathTo = pathToRelative & "\" & subNode.FullPath
                
                success = doCopyFile(pathFrom, pathTo)
                If success Then
                    loliteNode subNode
                End If
            End If
        End If
        
    Next

End Sub

Private Function hiliteNode(ByRef subNode As node)
    Const colorWorking = &HC0E0FF
    Const colorText = 0
    
    subNode.BackColor = colorWorking
    subNode.ForeColor = colorText
    subNode.EnsureVisible
    frmMain.trvFiles.Refresh
End Function

Private Function loliteNode(ByRef subNode As node)
    Const colorDone = &HC0FFC0
    
    subNode.BackColor = colorDone
    frmMain.trvFiles.Refresh
End Function

Public Sub calculateSize()
    Dim sizeAll As Long
    Dim subNode As node
    
    sizeAll = 0
    For Each subNode In frmMain.trvFiles.Nodes
        If subNode.Tag <> "folder" Then
            If selfAndParentsChecked(subNode) Then
                sizeAll = sizeAll + CLng(Val(subNode.Tag))
            End If
        End If
    Next
    
    frmMain.lblSizeAll.Caption = "Gre: " & formatByte(sizeAll)
End Sub

Private Function oneParentSelected(ByVal subNode As node) As Boolean
    Dim parentSelected As Boolean
    
    parentSelected = False
    Do
        If Not (subNode.Parent Is Nothing) Then
            Set subNode = subNode.Parent
            If subNode.Selected Then
                parentSelected = True
            End If
        Else
            Exit Do
        End If
    Loop
    
    oneParentSelected = parentSelected
End Function
 
Public Function selfAndParentsChecked(ByVal subNode As node)
    Dim parentNode As node
    Dim selfOrParentUnChecked As Boolean

    If subNode.Checked Then
        Do
            If Not (subNode.Parent Is Nothing) Then
                Set subNode = subNode.Parent
                If Not subNode.Checked Then
                    selfOrParentUnChecked = True
                    Exit Do
                End If
                
            Else
                Exit Do
            End If
        Loop
    Else
        selfOrParentUnChecked = True
    End If
    
    selfAndParentsChecked = Not selfOrParentUnChecked
End Function

Public Sub checkAllParents(ByVal subNode As node)
    Dim parentNode As node
    
    Do
        If Not (subNode.Parent Is Nothing) Then
            Set subNode = subNode.Parent
            subNode.Checked = True
        Else
            Exit Do
        End If
    Loop
    
End Sub

Private Sub doBuildSystemTree(ByRef tree As TreeView, ByVal topPath As String)
    Dim subNode As node
    Dim subFolder As Object
    Dim subFolderName As String
    Dim subFile As Object
    Dim folder As Object
    Dim doUse As Boolean
    
    If topPath <> "" Then
        Set mFileSystem = CreateObject("Scripting.FileSystemObject")
        
        Set folder = mFileSystem.getFolder(topPath)
        
        For Each subFolder In folder.subFolders
            subFolderName = subFolder.Name
            doUse = True
            If frmMain.chkExcludeFolder.Value Then
                doUse = Not (Left$(subFolderName, 1) = "_")
            End If
            If doUse Then
                Set subNode = tree.Nodes.Add(, , , subFolderName)
                subNode.Expanded = True
                subNode.Checked = True
                buildSystemTree tree, subFolder.path, subNode
                subNode.Expanded = False
            
                subNode.Tag = "folder"
            End If
        Next
        
        For Each subFile In folder.Files
            Set subNode = tree.Nodes.Add(, , , getFileName(subFile.Name))
            subNode.Checked = True
            
            subNode.Tag = subFile.Size
        Next
        
        Set mFileSystem = Nothing
    End If
   
End Sub

Private Sub buildSystemTree(ByRef tree As TreeView, ByVal topPath As String, ByRef treeNode As node)
    Dim folder As Object
    Dim subFolder As Object
    Dim subFile As Object
    Dim subNode As node

    Set folder = mFileSystem.getFolder(topPath)

    On Error GoTo buildSystemTreeError

    For Each subFolder In folder.subFolders
        Set subNode = tree.Nodes.Add(treeNode, tvwChild, , subFolder.Name)
        subNode.Expanded = True
        subNode.Checked = True
        subNode.EnsureVisible
        buildSystemTree tree, subFolder.path, subNode
        subNode.Expanded = False
        
        subNode.Tag = "folder"
    Next
    
    For Each subFile In folder.Files
        Set subNode = tree.Nodes.Add(treeNode, tvwChild, , getFileName(subFile.Name))
        subNode.Checked = True
        
        subNode.Tag = subFile.Size
    Next
    
    DoEvents
    frmMain.trvFiles.Refresh

    Exit Sub
buildSystemTreeError:
    If frmMain.gApplicationEnded Then
        End
    End If

End Sub
